@@ -4,6 +4,8 @@ from cppa3 import unify
 
 from cppa3.unify import unify
 
+from cppa3.profile import ChannelProfileHandler
+
 from inspect import getsourcefile
 from os.path import abspath, dirname, join
 
@@ -19,6 +21,7 @@ class CPPA3TestCase( unittest.TestCase ):
                             filename="cppa3_test.log")
         thisdir = dirname(abspath(getsourcefile(lambda:0)))
         self.testdatadir = join(thisdir,'data')
+        self.testconfigdir = join(thisdir,'config')
         if 'CPPA3XSDDIR' in os.environ:
             # If we have the location of the CPPA3 XSD,  use it to validate inputs and
             # outputs
@@ -30,26 +33,46 @@ class CPPA3TestCase( unittest.TestCase ):
             # Else no schema validation
             self.schema = None
 
+        self.parser = lxml.etree.XMLParser(remove_blank_text=True)
+
         # If you set the following to True, always generate CPAs
         # If False, preserve and check with previously created CPA
         self.replacemode = False
 
-    def do_unification(self, id):
+        self.configdatadir = join(thisdir,'config')
+        config_file = os.path.join(self.configdatadir,'channelprofiles.xml')
+        parser = lxml.etree.XMLParser(remove_blank_text=True)
+        configuration_data =  lxml.etree.parse(config_file, parser)
+        self.channel_profile_handler = ChannelProfileHandler(configuration_data).apply_profile_configs
+
+    def do_unification(self, id, agreementid=None,
+                       requested_activation_date=None,
+                       requested_expiration_date=None,
+                       acpp_url=None,
+                       bcpp_url=None,
+                       handle_defaults=False):
         logging.info('------------------------------------------')
         logging.info('Running test {}'.format(id))
         #logging.basicConfig(level=logging.DEBUG,
         #                    filename="cppa3_test.log")
         cppa_file = os.path.join(self.testdatadir,'cpp_a_'+id+'.xml')
         cppb_file = os.path.join(self.testdatadir,'cpp_b_'+id+'.xml')
-        acpp =  lxml.etree.parse(cppa_file)
-        bcpp =  lxml.etree.parse(cppb_file)
+        acpp =  (lxml.etree.parse(cppa_file, self.parser)).getroot()
+        bcpp =  (lxml.etree.parse(cppb_file, self.parser)).getroot()
         if self.schema != None:
             if self.schema.validate(acpp) and self.schema.validate(bcpp):
                 logging.info('CPP A and B are valid for {}'.format(id))
             else:
                 err = str(self.schema.error_log.last_error)
                 raise cppa3.unify.UnificationException('Invalid input: {}'.format(err))
-        cpa = unify(acpp, bcpp)
+        cpa = unify(acpp, bcpp, agreementid=agreementid,
+                    requested_activation_date=requested_activation_date,
+                    requested_expiration_date=requested_expiration_date,
+                    acpp_url=acpp_url,
+                    bcpp_url=bcpp_url,
+                    default_handler = self.channel_profile_handler,
+                    handle_defaults=handle_defaults
+                    )
         if self.schema is not None:
             if not self.schema.validate(cpa):
                 err = str(self.schema.error_log.last_error)
@@ -84,6 +107,9 @@ class CPPA3TestCase( unittest.TestCase ):
         logging.info('------------------------------------------')
 
 
+
+
+
     """
     General tests using NamedProtocolBinding
     """
@@ -194,6 +220,29 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_0031(self):
         self.do_unification('0031')
 
+    # test with explicitly set agreementid
+    def test_0040(self):
+        self.do_unification('0040', agreementid='our_agreement')
+
+    # test with activation date set
+    def test_0041(self):
+        import datetime
+        requested_date = now = datetime.datetime(2050,10,5,0,0,0)
+        self.do_unification('0041',  requested_activation_date=requested_date)
+
+    # test with expiration date set
+    def test_0042(self):
+        import datetime
+        requested_date = now = datetime.datetime(2050,10,5,0,0,0)
+        self.do_unification('0042',  requested_expiration_date=requested_date)
+
+    # test with CPP URLs
+    def test_0043(self):
+        self.do_unification('0043',
+                            acpp_url='http://a.example.com/acpp.xml',
+                            bcpp_url='http://b.example.com/bccp.xml')
+
+
     # Extra test, failing ActivationDate
     def test_0050(self):
         self.assertRaises( cppa3.unify.UnificationException, self.do_unification, '0210' )
@@ -253,6 +302,18 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_0107(self):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0107')
 
+
+    # Payload signature
+    def test_0110(self):
+        self.do_unification('0110')
+
+    # Payload signature and encryption
+    def test_0111(self):
+        self.do_unification('0111')
+
+    # Payload encryption
+
+
     """
     0200 ...  reserved for future use ...
     """
@@ -382,6 +443,68 @@ class CPPA3TestCase( unittest.TestCase ):
     """
 
 
+    """
+    Access Control feature
+
+    Allowed and denied parties, at various levels (CPP, ServiceBinding, ActionBinding).
+
+    """
+
+    # A sets allowed parties and Party B is one of them
+    def test_0700(self):
+        self.do_unification('0700')
+
+    # A sets allowed parties and Party B is not one of them
+    def test_0701(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0701')
+
+    # B sets allowed parties and Party A is one of them
+    def test_0702(self):
+        self.do_unification('0702')
+
+    # B sets allowed parties and Party A is not one of them
+    def test_0703(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0703')
+
+    # A sets denied parties but B is not one of them
+    def test_0704(self):
+        self.do_unification('0704')
+
+    # A sets denied parties and B is one of them
+    def test_0705(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0705')
+
+    # B sets denied parties but A is not one of them
+    def test_0706(self):
+        self.do_unification('0706')
+
+    # B sets denied parties and A is one of them
+    def test_0707(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0707')
+
+    # PartyIdListRef Variant of 0700
+    def test_0708(self):
+        self.do_unification('0708')
+
+    # PartyIdListRef Variant of 0705
+    def test_0709(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0709')
+
+    # Variant of 0008 where A blacklists B for one ServiceBinding
+    def test_0710(self):
+        self.do_unification('0710')
+
+    # Variant of 0008 where B blacklists A for one ServiceBinding
+    def test_0711(self):
+        self.do_unification('0711')
+
+    # Variant of 0008 where A blacklists B for one ServiceSpecification
+    def test_0712(self):
+        self.do_unification('0712')
+
+    # Variant of 0001 where A blacklists B for one Action
+    def test_0713(self):
+        self.do_unification('0713')
 
 
     """
@@ -398,6 +521,18 @@ class CPPA3TestCase( unittest.TestCase ):
         # two delivery channels,  the first one is non compatible so the second must be chosen
         self.do_unification('1001')
 
+    # Channel Profile;  specified on both sides, compatibly
+    def test_1002(self):
+        self.do_unification('1002')
+
+    # Channel Profile;  specified on both sides, incompatibly
+    def test_1003(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1003')
+
+    # Channel Profile;  specified on first side only
+    def test_1004(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1004')
+
     def test_1006(self):
         # two delivery channels,  the first one is non compatible so the second must be chosen
         self.do_unification('1006')
@@ -424,9 +559,6 @@ class CPPA3TestCase( unittest.TestCase ):
         # Variant of 1011 that uses channel features
         self.do_unification('1020')
 
-    # ENTSOG AS4 tests
-    # To do
-
     # MEPs
     # 1100 range
 
@@ -468,7 +600,6 @@ class CPPA3TestCase( unittest.TestCase ):
 
     # Two Way MEP inconsistent in referred to action
     def test_1123(self):
-        #self.do_unification('1123')
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1123')
 
     # Two Way MEP sync Send
@@ -529,6 +660,14 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_1147(self):
         self.do_unification('1147')
 
+    # Simple Push MEP Send with a two properties, in order, matching
+    def test_1148(self):
+        self.do_unification('1148')
+
+    # Simple Push MEP Send with a two properties, out of order, matching
+    def test_1149(self):
+        self.do_unification('1149')
+
     # Various MPC errors
     # Both Sender and Receiver set the MPC,  but incompatibly
     def test_1150(self):
@@ -543,6 +682,24 @@ class CPPA3TestCase( unittest.TestCase ):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1152')
 
 
+    # Property Sets.
+
+    # Two referenced property sets, compatible
+    def test_1160(self):
+        self.do_unification('1160')
+
+    # Two referenced property sets, incompatible
+    def test_1161(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1161')
+
+    # One referenced property set, one inline properties, compatible
+    def test_1162(self):
+        self.do_unification('1162')
+
+    # One referenced property set, one inline properties, incompatible
+    def test_1163(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1163')
+
     # 1200 range
     # Reliable Messaging:  AS4 reception awareness
 
@@ -605,6 +762,70 @@ class CPPA3TestCase( unittest.TestCase ):
         self.do_unification('1410')
 
 
+    # 1500, 1501, 1502, 1503 Multi Hop:  actorOrRole attribute present on none or on both,
+    # and if on both with same value
+    def test_1500(self):
+        self.do_unification('1500')
+
+    def test_1501(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1501')
+
+    def test_1502(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1502')
+
+    def test_1503(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1503')
+
+    # 1505 Addressing present, inline
+    def test_1504(self):
+        self.do_unification('1504')
+
+    # 1505 Addressing present in both but inconsistent Endpoint
+    def test_1505(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1505')
+
+    # 1505 Addressing in channel present in both but inconsistent Action
+    def test_1506(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1506')
+
+    # 1507 Reference parameter: ebMSInferredReverseRoutingInput
+    def test_1507(self):
+        self.do_unification('1507')
+
+    def test_1508(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1508')
+
+    def test_1509(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1509')
+
+    # Pull mode, with a routed PullRequest based on MPC, no RoutingInput
+    # See [EBMS3PART2], section 2.6.1, ad 2 (Signal Messages), first bullet
+    def test_1510(self):
+        self.do_unification('1510')
+
+    # Pull mode, with a routed PullRequest based on RoutingInput
+    # Reference parameter: ebMSInferredPullRequestRoutingInput
+    # See [EBMS3PART2], section 2.6.1, ad 2 (Signal Messages), second bullet
+    def test_1511(self):
+        self.do_unification('1511')
+
+    # Like 1511, but in addition there is a non-bundled error channel
+    def test_1512(self):
+        self.do_unification('1512')
+
+    # 1515 combination of sorts
+    def test_1515(self):
+        self.do_unification('1515')
+
+    # 1600 ENTSOG AS4 tests
+
+    # Not using ChannelProfile defaults
+    def test_1600(self):
+        self.do_unification('1600')
+
+    def test_1601(self):
+        self.do_unification('1601', handle_defaults=True)
+
 
     """
     Web Services Tests
